class GHLamps(object):
    """A container class to access the IoT Greenhouse lamps.
    """
    red = None
    white = None
    dual = None

    def __init__(self, pigpio, pi, red_led_gpio, white_led_gpio, dual_led_gpio1, dual_led_gpio2):
        """Initializes a container for IoT Greenhouse lamps
        
        :param pi: reference to pigpio service
        :param red_led_gpio: GPIO port for red LED. Use GHgpio definitions.
        :param white_led_gpio: GPIO port for white LED. Use GHgpio definitions.
        :param dual_led_gpio1: GPIO port for dual LED pin 1. Use GHgpio definitions.
        :param dual_led_gpio2: GPIO port for dual LED pin 2. Use GHgpio definitions.
        :returns: GHLamps object
        """
        self.red = GHLamp(pigpio, pi, red_led_gpio)
        self.white =  GHLamp(pigpio, pi, white_led_gpio)
        self.dual = GHLampDual(pigpio, pi, dual_led_gpio1, dual_led_gpio2)


class GHLamp(object):
    """A class to access IoT Greenhouse LED lamp.
    """   
    LAMP_ON = False #active low
    LAMP_OFF = True

    def __init__(self, pigpio, pi, gpio):
        """Initializes an IoT Greenhouse lamp.
        
        :param pi: reference to pigpio service
        :param gpio: GPIO port for LED. Use GHgpio definitions.
        :returns: GHLamp object
        """
        self._pi = pi
        self._gpio = gpio
        self._pi.set_mode(self._gpio, pigpio.OUTPUT) 
        self._pi.write(self._gpio, self.LAMP_OFF)

    def on(self):
        """Activate lamp.
        """
        self._pi.write(self._gpio, self.LAMP_ON)
    
    def off(self):
        """Deactivate lamp.
        """
        self._pi.write(self._gpio, self.LAMP_OFF)

    def toggle(self):
        """Toggle lamp state.
        """
        if self.is_on():
            self.off()
        else:
            self.on()

    def get_state(self):
        """Returns current state of lamp.
        """
        return self._pi.read(self._gpio)
    
    def is_on(self):
        """Returns Boolean indicating lamp is on.
        """
        return self.get_state() == self.LAMP_ON

    def is_off(self):
        """Returns Boolean indicating lamp is off.
        """
        return self.get_state() == self.LAMP_OFF
    
    def get_status(self):
        """Gets status of lamp.

        :returns: Lamp status as string.
        """
        if self.is_on():
            return "ON"
        else:
            return "OFF"
            

class GHLampDual(object):
    """A class to access the IoT Greenhouse dual color lamp.
    """   
    LAMP_OFF = 0
    LAMP_ON_YELLOW = 1
    LAMP_ON_GREEN = 2

    def __init__(self, pigpio, pi, shared_gpio1, gpio2):
        """Initializes an IoT Greenhouse dual lamp.
        
        :param pi: reference to pigpio service
        :param shared_gpio1: GPIO port 1 for Dual LED. Currently shared with RED LED.
        :param gpio2: GPIO port 2 for Dual LED. 
        :returns: GHLampDual object
        """
        self.pigpio = pigpio
        self._pi = pi
        self._shared_gpio1 = shared_gpio1
        self._gpio2 = gpio2
        self._pi.set_mode(self._shared_gpio1, self.pigpio.OUTPUT)   
        self._pi.set_mode(self._gpio2, pigpio.INPUT) 
        self._pi.write(self._shared_gpio1, True)
        
        self._state = self.LAMP_OFF

    def on_green(self):
        """Activates dual lamp as green.
        """
        self._pi.set_mode(self._gpio2, self.pigpio.OUTPUT)
        self._pi.write(self._shared_gpio1, False)
        self._pi.write(self._gpio2, True)
        self._state = self.LAMP_ON_GREEN

    def on_yellow(self):
        """Activates dual lamp as yellow.
        """
        self._pi.set_mode(self._gpio2, self.pigpio.OUTPUT)
        self._pi.write(self._shared_gpio1, True)
        self._pi.write(self._gpio2, False)
        self._state = self.LAMP_ON_YELLOW

    def off(self):
        """Deactivates dual lamp.
        """
        self._pi.set_mode(self._gpio2, self.pigpio.INPUT)
        self._state = self.LAMP_OFF

    def get_state(self):
        """Returns current state of dual lamp

        :returns: GHLampDual.LAMP_ON_GREEN, GHLampDual.LAMP_ON_YELLOW, or GHLampDual.LAMP_OFF
        """
        return self._state

    def is_green(self):
        """Returns Boolean indicating lamp is green.
        """
        return self.get_state() == self.LAMP_ON_GREEN

    def is_yellow(self):
        """Returns Boolean indicating lamp is yellow.
        """
        return self.get_state() == self.LAMP_ON_YELLOW

    def is_off(self):
        """Returns Boolean indicating lamp is off.
        """
        return self.get_state() == self.LAMP_OFF

    def get_status(self):
        """Gets status of dual lamp.

        :returns: Button status as string.
        """
        if self.is_green():
            return "GREEN"
        if self.is_yellow():
            return "YELLOW"
        else:
            return "OFF"


from tarfile import RECORDSIZE
from typing import OrderedDict
import requests
import json

class AirTable:
        
    def __init__(self, ws_url, base_key, api_key, table_name):
        self.ws_url = ws_url
        self.base_key = base_key
        self.api_key = api_key
        self.table_name = table_name

    def get_records(self) -> list:
        ''' Returns a list of record dictionaries'''
        url = f"{self.ws_url}{self.base_key}/{self.table_name}"
        data = self._get_data(url)
        records = data["records"]
        return records

    def get_record(self, id:str) -> dict:
        ''' Returns a single record dictionary'''
        url = f"{self.ws_url}{self.base_key}/{self.table_name}/{id}"
        record = self._get_data(url)
        return record

    def get_records_by_search(self, field:str, value:str) -> list:
        ''' Returns a list of record dictionaries based on field and value'''
        item = None
        url = f"{self.ws_url}{self.base_key}/{self.table_name}"
        filter = f"SEARCH(%22{value}%22%2C%7B{field}%7D)"
        param = f"filterByFormula={filter}"
        data = self._get_data(url, param)
        records = data["records"]
        return records
  
    def add_record(self, fields:dict) -> dict:
        ''' Adds a record'''
        data = json.dumps({"fields": fields})
        url = f"{self.ws_url}{self.base_key}/{self.table_name}"
        headers = {"Authorization": f"Bearer {self.api_key}",
        "Content-Type": "application/json"}
        params = None
 
        r = requests.post(url=url, headers=headers, params=params, data=data)
    
        if r.status_code != 200:
            message = f"Bad request. Unable to create record. " + r.text
            if self.log_service:
                self.log_service.update("ERROR", message)
            raise Exception(message)
        else:
            record = r.json()
            return record
            
    def update_record(self, record:dict) -> dict:
        ''' Updates a record'''
        data = json.dumps({"fields":record["fields"]})
        
        url = f"{self.ws_url}{self.base_key}/{self.table_name}/{record['id']}"
        headers = {"Authorization": f"Bearer {self.api_key}",
        "Content-Type": "application/json"}
        params = None
 
        r = requests.patch(url=url, headers=headers, params=params, data=data)
    
        if r.status_code != 200:
            message = f"Bad request. Unable to update record. " + r.text
            if self.log_service:
                self.log_service.update("ERROR", message)
            raise Exception(message)
        else:
            record = r.json()
            # record["id"] = r.json()["id"]
            # record["fields"] = r.json()["fields"]
            return record

    def delete_record(self, record:dict):
        ''' Deletes a record'''
        url = f"{self.ws_url}{self.base_key}/{self.table_name}/{record['id']}"
        headers = {"Authorization": f"Bearer {self.api_key}"}
        params = None
        r = requests.delete(url=url, headers=headers, params=params)
        
        if r.status_code != 200:
            message = f"Bad request. Unable to delete record. " + r.text
            if self.log_service:
                self.log_service.update("ERROR", message)
            raise Exception(message)


    def _get_data(self, url, params = None): 
        data = None
        headers = {"Authorization": f"Bearer {self.api_key}"}
        r = requests.get(url=url, headers=headers, params=params) #, verify=False)
        if r.status_code == 404:
            pass #okay return None, id is not found
        elif r.status_code != 200:
            message = f"Bad request. Unable to fetch report data. " + r.text
            if self.log_service:
                self.log_service.update("ERROR", message)
            raise Exception(message)
        else:
            data = r.json()
        return data







import socket
class IoTGreenhouse(object):
    """IoT Greenhouse data object"""
    def __init__(self, id=None, name=None): 
        self.id = id
        host_name = socket.gethostname()
        if "Greenhouse" in host_name:
            self.name = host_name
            self.group = self.name[-4:-2]
            self.number = self.name[-2:]
        elif name:
            self.name = name
            self.group = "xx"
            self.number = "00"
        else:
            self.name = "testGH"
            self.group = "xx"
            self.number = "00"
        #house state
        self.led_red_state = None
        self.led_red_status = None
        self.led_white_state = None
        self.led_white_status = None
        self.led_dual_state = None
        self.led_dual_status = None
        self.switch_pb_state = None
        self.switch_pb_status = None
        self.switch_toggle_state = None
        self.switch_toggle_status = None
        self.fan_state = None
        self.fan_status = None
        self.servo_position = None
        self.servo_status = None
        self.heater_state = None
        self.heater_status = None
        self.buzzer_state = None
        self.buzzer_status = None
        self.ain_pot_raw = None
        self.ain_light_raw = None
        self.ain_aux_raw = None
        self.temp_inside_C = None
        self.temp_inside_F = None
        self.temp_outside_C = None
        self.temp_outside_F = None
        self.last_update = None
        self.message = None

    



class GHFan(object):
    """A class to access the IoT Greenhouse fan.
    """
    
    FAN_ON = True
    FAN_OFF = False

    def __init__(self, pigpio, pi, fan_gpio):
        """Initializes the IoT Greenhouse fan.
        
        :param pi: reference to pigpio service
        :param fan_gpio: GPIO port for fan. Use GHgpio definitions.
        :returns: GHFan object
        """
        self._pi = pi
        self._gpio = fan_gpio
            
        self._pi.set_mode(self._gpio, pigpio.OUTPUT)        
        #Fan off
        self._pi.write(self._gpio, self.FAN_OFF)
        
    def on(self):
        """Activates the fan.
        """
        self._pi.write(self._gpio, self.FAN_ON)
        
    def off(self):
        """Deactivates the fan.
        """
        self._pi.write(self._gpio, self.FAN_OFF)
    
    def toggle(self):
        """Toggle fan state.
        """
        if self.is_on():
            self.off()
        else:
            self.on()

    def get_state(self):
        """Returns the current state of the fan.
        """
        return self._pi.read(self._gpio)

    def is_on(self):
        """Returns Boolean indicating fan is on.
        """
        return self.get_state() == self.FAN_ON

    def is_off(self):
        """Returns Boolean indicating fan is off.
        """
        return self.get_state() == self.FAN_OFF
    
    def get_status(self):
        """Gets status of fan.

        :returns: Fan status as string.
        """
        if self.get_state() == self.FAN_ON:
            return "ON"
        else:
            return "OFF"


import datetime
from iot_gh.src.iot_gh.GHgpio import GHgpio
from iot_gh.src.iot_gh.GHAnalogService import GHAnalogService
from iot_gh.src.iot_gh.GHBuzzer import GHBuzzer
from iot_gh.src.iot_gh.GHFan import GHFan
from iot_gh.src.iot_gh.GHLamps import GHLamps
from iot_gh.src.iot_gh.GHServo import GHServo
from iot_gh.src.iot_gh.GHSwitches import GHSwitches
from iot_gh.src.iot_gh.GHTemperature import GHTemperature
from iot_gh.src.iot_gh.GHWebService import GHWebService
from iot_gh.src.iot_gh.IoTGreenhouse import IoTGreenhouse

class GHService(object):
    """A container class for IoT Greenhouse services.
    
    References to component services to update IoT Greenhouse state
    """
    def __init__(self, pigpio, pi, spi, conf):
        """Initializes an IoT Greenhouse container.
        """
        self.pigpio = pigpio
        self._pi = pi
        self._spi = spi

        if not self._pi.connected:
            raise Exception("ERROR: unable to connect to pigpio")
 

        #Makes component services for container.
        self.analog = GHAnalogService(self._pi, self._spi)
        self.buzzer = GHBuzzer(self.pigpio, self._pi, GHgpio.BUZZER)
        self.fan = GHFan(self.pigpio, self._pi, GHgpio.FAN)
        self.lamps = GHLamps(self.pigpio, self._pi, GHgpio.RED_LED, GHgpio.WHITE_LED, GHgpio.RED_LED, GHgpio.DUAL_LED)
        self.servo = GHServo(self.pigpio, self._pi, GHgpio.SERVO_PWM, conf.servo_cw_position, conf.servo_ccw_position)
        self.switches = GHSwitches(self.pigpio, self._pi, GHgpio.SWITCH_PB, GHgpio.SWITCH_TOGGLE)
        self.temperature = GHTemperature(self.analog.aux, self.analog.temp)
        self.web_service = GHWebService(self, conf.api_key, conf.base_key)    
        
    def update_greenhouse(self, greenhouse:IoTGreenhouse ):
        """Updates IoTGreenhouse object by reading all house 
        states and refreshing gh object.
        """
        try:
    
            greenhouse.led_red_state = self.lamps.red.get_state()
            greenhouse.led_red_status = self.lamps.red.get_status()
            greenhouse.led_white_state = self.lamps.white.get_state()
            greenhouse.led_white_status = self.lamps.white.get_status()
            greenhouse.led_dual_state =  self.lamps.dual.get_state()
            greenhouse.led_dual_status = self.lamps.dual.get_status()
            greenhouse.switch_pb_state = self.switches.push_button.get_state()
            greenhouse.switch_pb_status = self.switches.push_button.get_status()
            greenhouse.switch_toggle_state = self.switches.toggle.get_state()
            greenhouse.switch_toggle_status = self.switches.toggle.get_status()
            greenhouse.fan_state =  self.fan.get_state()
            greenhouse.fan_status =  self.fan.get_status()
            greenhouse.servo_position = self.servo.get_value()
            greenhouse.servo_status = self.servo.get_status()
            greenhouse.heater_state = self.lamps.white.get_state()
            greenhouse.heater_status = self.lamps.white.get_status()
            greenhouse.buzzer_state = self.buzzer.get_state()
            greenhouse.buzzer_status = self.buzzer.get_status()
            greenhouse.ain_pot_raw = self.analog.pot.get_value()
            greenhouse.ain_light_raw =  self.analog.light.get_value()
            greenhouse.ain_aux_raw =  self.analog.aux.get_value()

            greenhouse.temp_inside_C = self.temperature.get_inside_temp_C()
            greenhouse.temp_inside_F = self.temperature.get_inside_temp_F()
            greenhouse.temp_outside_C = self.temperature.get_outside_temp_C()
            greenhouse.temp_outside_F = self.temperature.get_outside_temp_F()
            
            greenhouse.last_update = datetime.datetime.now().replace(microsecond=0).isoformat(' ')
            
            self._make_message(greenhouse)
            
        except Exception as e:
            greenhouse.message = "Update exception: %s" % e.args[0]
            
    def _make_message(self, gh:IoTGreenhouse):
        m = ""
        if  gh.temp_outside_C > gh.temp_inside_C: 
            m = "Warning: The temperature outside the greenhouse is higher than the internal temperature."
        elif gh.servo_status == "CLOSED" and gh.fan_status == "ON":
            m = "Error: The fan is activated but are closed."
        else:
            m = "Greenhouse status is normal."

        gh.message = m


class GHAnalogService(object):
    """A container class to access IoT Greenhouse analog channels

    Provides collection of 8 A/D inputs based on channel definitions.
    """
    # Define sensor channels
    POT_CHANNEL  = 0
    LIGHT_CHANNEL  = 1
    TEMP_CHANNEL = 2
    AUX_CHANNEL = 3
    AD_IN_4_CHANNEL = 4
    AD_IN_5_CHANNEL = 5
    AD_IN_6_CHANNEL = 6
    AD_IN_7_CHANNEL = 7

    def __init__(self, pi, spi):
        """Provides access to IoT Greenhouse analog channels
        
        :param pi pigpio.pi: Reference to current pigpio service
        :param spi spidev.spi: Reference to spi object
        :returns: AnalogService container object
        """
        self._pi = pi
        # Open SPI bus
        self._spi = spi 
        self._spi.open(0,0)
        self._spi.max_speed_hz=1000000
        
        self.pot = GHAnalogChannel(self._spi, GHAnalogService.POT_CHANNEL)
        self.light = GHAnalogChannel(self._spi, GHAnalogService.LIGHT_CHANNEL)
        self.temp = GHAnalogChannel(self._spi, GHAnalogService.TEMP_CHANNEL)
        self.aux = GHAnalogChannel(self._spi, GHAnalogService.AUX_CHANNEL)
        self.ad_in_4 = GHAnalogChannel(self._spi, GHAnalogService.AD_IN_4_CHANNEL)
        self.ad_in_5 = GHAnalogChannel(self._spi, GHAnalogService.AD_IN_5_CHANNEL)
        self.ad_in_6 = GHAnalogChannel(self._spi, GHAnalogService.AD_IN_6_CHANNEL)
        self.ad_in_7 = GHAnalogChannel(self._spi, GHAnalogService.AD_IN_7_CHANNEL)
 
 
class GHAnalogChannel(object):
    """A class to access analog channels. Provides get functions to 
    read both raw digital and voltage values.
    """

    def __init__(self, spi, channel):
        """Initializes an analog channel.
        
        :param spi spidev.spi: Reference to spi object
        :param channel int: Channel index for this channel
        :returns: GHAnalogChannel service for specified channel
        """
        self._spi = spi  
        self._channel = channel
        
    def get_value(self):
        """Gets raw digital value for this analog channel
        
        :return: Digital value of analog channel
        """
        adc = self._spi.xfer2([1,(8+self._channel)<<4,0])
        data = ((adc[1]&3) << 8) + adc[2]
        return data

    # rounded to specified number of decimal places.
    def get_voltage(self, places = 1):
        """Gets voltage value for this analog channel
        
        :param places int: Number of places to round to. Default is 1.
        :return: Voltage value of analog channel
        """
        data = self.get_value()   
        volts = (data * 3.3) / float(1023)
        volts = round(volts, places)
        return volts




import time
from iot_gh.src.iot_gh.IoTGreenhouse import IoTGreenhouse
from iot_gh.src.iot_gh.AirTable import AirTable

class GHWebService(object):
    """ Web connector for REST service. 
    Post data using json package.
    V3
    10/16/2022
    """
    POST_DELAY = 5     #Seconds between posts - throttle
    DATA_TABLE_NAME = 'GreenhouseData'
    LOGS_TABLE_NAME = 'GreenhouseLogs'
    
    def __init__(self, ws_url, api_key, base_key):
        self.ws_url = ws_url
        self.api_key = api_key
        self.base_key = base_key
        self.data_table = AirTable(ws_url=ws_url,base_key=base_key, api_key=api_key,table_name=self.DATA_TABLE_NAME)
        self.logs_table = AirTable(ws_url=ws_url,base_key=base_key, api_key=api_key,table_name=self.LOGS_TABLE_NAME)
        self.last_post_time = time.time() - 3

    def get_greehhouse(self, id:str=None, name:str=None) -> IoTGreenhouse:
        if id:
            record = self.data_table.get_record(id=id)
        elif name:
            records = self.data_table.get_records_by_search("Name", name)
            record = records[0]
        else:
            return None
        
        gh = IoTGreenhouse()
        self._fill(gh, record)
        return gh
        
    def post_greenhouse(self, gh:IoTGreenhouse, throttle=True):
        #throttle post
        if not throttle or time.time() > self.last_post_time + self.POST_DELAY:
            if gh.id:
                record = self._make_record(gh)
                record = self.data_table.update_record(record)
            else:
                fields = self._make_fields(gh)
                record = self.data_table.add_record(fields)

            self._fill(gh, record)
            self.last_post_time = time.time()      

    def delete_greenhouse(self, gh:IoTGreenhouse):
        if gh.id:
            record = self._make_record(gh)
            self.data_table.delete_record(record)


    def _make_record(self, gh:IoTGreenhouse) -> dict:
        record = {"id": gh.id, "fields": self._make_fields(gh)}
        return record

    def _make_fields(self, gh:IoTGreenhouse) -> dict:    
        fields = {}
        if gh.name:
            fields["Name"] = gh.name
        if gh.temp_inside_F:
            fields["Temp In"] = gh.temp_inside_F
        if gh.temp_outside_F:
            fields["Temp Out"] = gh.temp_outside_F
        if gh.servo_status:
            fields["Servo Status"] = gh.servo_status
        if gh.fan_status:
            fields["Fan Status"] = gh.fan_status
        if gh.led_white_status:
            fields["Heater Status"] = gh.led_white_status
        if gh.message:
            fields["Message"] = gh.message
        return fields

    def _update_record(self, record:dict, gh:IoTGreenhouse):
        record["Temp In"] = gh.temp_inside_F,
        record["Temp Out"] = gh.temp_outside_F,
        record["Servo Status"] = gh.servo_status,
        record["Fan Status"] = gh.fan_status,
        record["Heater Status"] = gh.led_white_status,  #white led simulates heater
        record["Message"] = gh.message
        
    def _fill(self, gh:IoTGreenhouse, record:dict) -> None:
        gh.id = record["id"]
        gh.ID = record["fields"].get("ID")
        gh.name =  record["fields"].get("Name")
        gh.temp_inside_F = record["fields"].get("Temp In")
        gh.temp_outside_F = record["fields"].get("Temp Out")
        gh.servo_status = record["fields"].get("Servo Status")
        gh.fan_status = record["fields"].get("Fan Status")
        gh.led_white_status = record["fields"].get("Heater Status")
        gh.message = record["fields"].get("Message")


class GHServo(object):
    """A class to access the IoT Greenhouse servo.
    """    
    FREQ = 50
    PERIOD = 1/FREQ
    CCW_PW_LIMIT = 2000
    CW_PW_LIMIT = 500


    # _ccw_pulse_width = CCW_PW_LIMIT
    # _cw_pulse_width = CW_PW_LIMIT
   
    # _pulse_width = CCW_PW_LIMIT
    
    
    def __init__(self, pigpio, pi, servo_gpio,  cw_position, ccw_position):
        """Initializes the IoT Greenhouse servo.
        
        :param pi: reference to pigpio service
        :param servo_gpio: GPIO PWM port for servo. Must be hardware PWM pin
        :param ccw_position: Counter-clockwise pulse width in microseconds
        :param cw_position: Clockwise pulse width in microseconds
        :returns: GHServo object
        """
        #cw and ccw position in microseconds
        if cw_position < self.CW_PW_LIMIT:
            raise ValueError("Error: CW position value must be between greater than %i" % self.CW_PW_LIMIT)
        elif cw_position > self.CCW_PW_LIMIT:
            raise ValueError("Error: CW position value must be between less than %i" % self.CCW_PW_LIMIT)
        if servo_gpio != 18:
            raise ValueError("Error: Only GPIO18 currently supported.")
  
        self._pi = pi
        self._gpio = servo_gpio  
        self._ccw_pulse_width = ccw_position
        self._cw_pulse_width = cw_position
        self._center_pulse_width = (cw_position + ccw_position)//2        
        
        #pi.hardware_PWM(self._gpio, self.FREQ, 250000) # 800Hz 25% dutycycle
        self._position = 0
        dc = self._calc_duty_cycle(self._calc_pulse_width(self._position))
        self._pi.hardware_PWM(self._gpio, self.FREQ, dc)       
        
    def _calc_pulse_width(self, position):
        if position < 0 or position > +1:
            raise ValueError("Error: Servo position is specified between 0 (full CCW) and 1 (full CW).")
        
        pw = self._ccw_pulse_width - position * (self._ccw_pulse_width - self._cw_pulse_width)
        return pw

    def _calc_duty_cycle(self, pulse_width):
        if pulse_width < self._cw_pulse_width or pulse_width > self._ccw_pulse_width:
            raise ValueError("Error: Servo pulse width is specified between %i (full CCW) and %i (full CW)." % (self._ccw_pulse_width, self._cw_pulse_width))
        
        dc = int(pulse_width/self.PERIOD)
        return dc

    def move(self, position=0):
        """ Normalized move method with 0 being max left (ccw) and +1 being max right (cw)
        
        :param position: Normalized position value between 0 and +1.0.
        """
        if position < 0 or position > 1:
            raise Exception("Error: Position values must be between 0 and 1.0.")
        else:
            self._position = position
            dc = self._calc_duty_cycle(self._calc_pulse_width(self._position))
            self._pi.hardware_PWM(self._gpio, self.FREQ, dc)       

    def is_ccw(self):
        """Returns Boolean indicating servo is fully counter-clockwise.
        """
        return self._position == 0

    def is_cw(self):
        """Returns Boolean indicating servo is fully clockwise.
        """
        return self._position == +1

    def get_value(self):
        """Returns current position of servo.
        """
        return self._position

    def get_status(self):
        """Gets status of servo.

        :returns: Servo status as string.
        """
        if self.is_ccw():
            return "CLOSED"
        elif self.is_cw():
            return "OPEN"
        else:
            return "ACTIVE"

    
def test_servo():
    # pi = pigpio.pi()
    pi = pi()
    s = GHServo(pi, GHgpio.SERVO_PWM, 1000, 2000)
    print("move to 0")
    s.move(0)
    sleep(3)
    print("move to +1")
    s.move(+1)
    sleep(3)
    print("move to 0")
    s.move(0)
    sleep(3)
    print("move to +1")
    s.move(+1)
    sleep(3)

if __name__ == "__main__":
    test_servo()





''' Read config file to provide GHConf object
'''
import json

class GHConf():
    CONF_FILE = "/home/gh/.gh/gh_conf.json"

    def __init__(self):
        self.json = self._get_conf()

    @property
    def version(self) -> str:
        return self.json["gh_conf"]["version"]

    @property
    def servo_cw_position(self) -> str:
        return self.json["gh_conf"]["servo_cw_position"]

    @property
    def servo_ccw_position(self) -> str:
        return self.json["gh_conf"]["servo_ccw_position"]

    @property
    def ws_url(self) -> str:
        return self.json["gh_conf"]["ws_url"]

    @property
    def api_key(self) -> str:
        return self.json["gh_conf"]["api_key"]

    @property
    def base_key(self) -> str:
        return self.json["gh_conf"]["base_key"]

    def _get_conf(self) -> dict:
        with open(self.CONF_FILE ) as json_data_file:
            data = json.load(json_data_file)
        return data

    def _set_conf(self, data:dict):
        with open(self.CONF_FILE , "w") as outfile:
            json.dump(data, outfile)




import configparser
from iot_gh.GHService import GHService
from iot_gh.GHConf import GHConf

class IoTGreenhouseService(GHService):
    """A container class for IoT Greenhouse services implemented on the 
    Raspberry Pi.
    """

    def __init__(self):
        import os
        import pigpio
        import spidev
        import configparser

        pi = pigpio.pi()
        spi = spidev.SpiDev()
        conf = GHConf()
        super().__init__(pi, spi, conf)

